--[[
Name: Where's He Gone Now?
Revision: $Revision: 19 $
Author(s): ThePants999
Website: http://http://www.wowace.com/addons/wheres-he-gone-now/
SVN: svn://svn.wowace.com/wow/wheres-he-gone-now/mainline/trunk
Description: Keeps an eye on someone's distance from you, and alerts you if they get too far.
License: GNU General Public License version 3 (GPLv3)
]]

-- State

local WarningActive = false
local TomTomWaypoint = nil
local Arrow = nil


-- Initialization

local AppName = "Where's He Gone Now?"
local VERSION = AppName .. " -r" .. ("$Revision: 19 $"):match("%d+")

WHGN = LibStub("AceAddon-3.0"):NewAddon(AppName, "AceConsole-3.0", "AceTimer-3.0")



-- Callbacks

function WHGN:SaveArrowPosition(eventName, someRandomParam, point, parent, relpoint, x, y)
  if Arrow then
    self.db.char.ArrowPosSaved = true
    self.db.char.ArrowPoint = point
    self.db.char.ArrowParent = parent
    self.db.char.ArrowRelPoint = relpoint
    self.db.char.ArrowX = x
    self.db.char.ArrowY = y
  end
end



-- Other libraries

local mapdata = LibStub("LibMapData-1.0")
local LA = LibStub("LibArrow-1.0")
LA.RegisterCallback(WHGN, "MOVED", "SaveArrowPosition")


-- Config functions

function WHGN:GetEnabled(info)
  return self.db.char.Enabled
end

function WHGN:SetEnabled(info, input)
  if input then
    self.db.char.Enabled = true
  else
    self.db.char.Enabled = false
  end
  
  if self.db.char.Enabled then
    self:OnEnable(false)
  else
    self:OnDisable()
  end
end

function WHGN:GetInterval(info)
  return self.db.char.UpdateInterval
end

function WHGN:SetInterval(info, input)
  self.db.char.UpdateInterval = input
  self:OnDisable()
  self:OnEnable(false)
end

function WHGN:GetChar(info)
  return self.db.char.PlayerName
end

function WHGN:SetChar(info, input)
  self.db.char.PlayerName = input
  WarningActive = false
end

function WHGN:GetRange(info)
  return self.db.char.MaxRange
end

function WHGN:SetRange(info, input)
  self.db.char.MaxRange = input
  WarningActive = false
end



-- Config options

local OptionsTable = {
  handler = WHGN,
  type = "group",
  args = {
    enable = {
      name = "Enabled",
      desc = "Enable or disable the addon.",
      type = "toggle",
      get = "GetEnabled",
      set = "SetEnabled",
    },
    interval = {
      name = "Update interval",
      desc = "How often should the addon check the range? Lower values mean you find out faster if they're on the wander, but the addon uses more CPU.",
      type = "range",
      min = 0.1,
      max = 3.0,
      step = 0.1,
      get = "GetInterval",
      set = "SetInterval",
    },
    char = {
      name = "Character name",
      desc = "Which character do you want to track?",
      type = "input",
      get = "GetChar",
      set = "SetChar",
    },
    range = {
      name = "Maximum range",
      desc = "How far away will you allow them to get before you get a warning?",
      type = "range",
      min = 20,
      max = 79,
      step = 1,
      get = "GetRange",
      set = "SetRange",
    },
  }
}

LibStub("AceConfig-3.0"):RegisterOptionsTable(AppName, OptionsTable, {})
local OptionsDialog = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(AppName, AppName)

function WHGN:ShowOptionsDialog()
  InterfaceOptionsFrame_OpenToCategory(OptionsDialog)
end

-- AceAddon functions

function WHGN:OnInitialize()
  -- Set defaults
  local defaults = {
    char = {
      Enabled = true,
      PlayerName = "Daftagnan",
      MaxRange = 55,
      UpdateInterval = 0.5,
    }
  }
      
  self.db = LibStub("AceDB-3.0"):New("WHGN_DB", defaults)  
  self:Print("Tracking " .. self.db.char.PlayerName .. ", maximum range " .. self.db.char.MaxRange)
end

function WHGN:OnEnable(first)
  self:RegisterChatCommand("whgn", "ShowOptionsDialog")
  if self.db.char.Enabled then
    self.timer = self:ScheduleRepeatingTimer("TimerPop", self.db.char.UpdateInterval)
  end
    
  if self.db.char.ArrowPosSaved then
    Arrow = LA:NewArrow(self.db.char.ArrowPoint, self.db.char.ArrowParent, self.db.char.ArrowRelPoint, self.db.char.ArrowX, self.db.char.ArrowY)
  else
    Arrow = LA:NewArrow()
  end
  
  WarningActive = false  
end

function WHGN:OnDisable()
  self:UnregisterChatCommand("whgn")
  self:CancelAllTimers()
  Arrow:Release()
  if TomTomWaypoint then
    TomTom:RemoveWaypoint(TomTomWaypoint)
  end
end



-- Real functionality!

function WHGN:TimerPop()
  local x, y = GetPlayerMapPosition(self.db.char.PlayerName)
  local map = GetCurrentMapAreaID()
  local floor = GetCurrentMapDungeonLevel()
  
  if WarningActive then
    -- We've already warned the player. Check to see if we're OK again.
    
    if (UnitInParty(self.db.char.PlayerName) or UnitInRaid(self.db.char.PlayerName)) and (UnitAffectingCombat("player") or UnitAffectingCombat(self.db.char.PlayerName)) then
      -- Still in combat; range is relevant. Start off assuming we're OK, in order to switch
      -- off the warning if we can no longer figure out the range (e.g. because we're in a
      -- different zone).
      local range, direction = 0, 0
      
      if x and x > 0 and y and y > 0 and map then
        -- We have the target's position.
        range, direction = mapdata:DistanceAndDirection(map, floor, x, y)
      end
      
      if range <= self.db.char.MaxRange then
        -- Back within range, phew.
        WarningActive = false
      end
    else
      -- No longer in combat, or no longer grouped.
      WarningActive = false
    end
  else
    -- Warning currently inactive. Do we now have a problem?
    
    if (UnitInParty(self.db.char.PlayerName) or UnitInRaid(self.db.char.PlayerName)) and (UnitAffectingCombat("player") or UnitAffectingCombat(self.db.char.PlayerName)) then
      -- In combat; range is relevant. Start off assuming we're OK, in order not to switch
      -- on the warning if we can't  figure out the range (e.g. because we're in a
      -- different zone).
      local range, direction = 0, 0
      
      if x and x > 0 and y and y > 0 and map then
        -- We have the target's position.
        range, direction = mapdata:DistanceAndDirection(map, floor, x, y)
      end
      
      if range > self.db.char.MaxRange then
        -- Noooo! Out of range!
        self:Print(self.db.char.PlayerName .. " is getting away!")
        PlaySound("TellMessage")
        WarningActive = true
      end
    end
  end
  
  if TomTomWaypoint then
    TomTom:RemoveWaypoint(TomTomWaypoint)
  end
  if not WarningActive then
    Arrow:Hide()
  end
  
  -- If TomTom is available, we'll use that. Otherwise, set up an internal arrow.
  if WarningActive then
    if TomTom then
      local zone = GetCurrentMapZone()
      local continent = GetCurrentMapContinent()
      TomTomWaypoint = TomTom:AddZWaypoint(
        continent,
        zone,
        x * 100,
        y * 100,
        self.db.char.PlayerName,
        false,
        true,
        false,
        nil,
        true,
        true)
    else
      Arrow:SetWaypoint(map, floor, x, y)
      Arrow:SetTitleText(self.db.char.PlayerName)
      Arrow:Show()
    end
  end
end